@@ -4,27 +4,24 @@ Module for formatting output data into CSV files.
 
 from __future__ import annotations
 
+from collections.abc import (
+    Hashable,
+    Iterable,
+    Iterator,
+    Sequence,
+)
 import csv as csvlib
 import os
 from typing import (
     TYPE_CHECKING,
     Any,
-    Hashable,
-    Iterator,
-    Sequence,
     cast,
 )
 
 import numpy as np
 
 from pandas._libs import writers as libwriters
-from pandas._typing import (
-    CompressionOptions,
-    FilePathOrBuffer,
-    FloatFormatType,
-    IndexLabel,
-    StorageOptions,
-)
+from pandas.util._decorators import cache_readonly
 
 from pandas.core.dtypes.generic import (
     ABCDatetimeIndex,
@@ -39,16 +36,28 @@ from pandas.core.indexes.api import Index
 from pandas.io.common import get_handle
 
 if TYPE_CHECKING:
+    from pandas._typing import (
+        CompressionOptions,
+        FilePath,
+        FloatFormatType,
+        IndexLabel,
+        StorageOptions,
+        WriteBuffer,
+    )
+
     from pandas.io.formats.format import DataFrameFormatter
 
 
+_DEFAULT_CHUNKSIZE_CELLS = 100_000
+
+
 class CSVFormatter:
     cols: np.ndarray
 
     def __init__(
         self,
         formatter: DataFrameFormatter,
-        path_or_buf: FilePathOrBuffer[str] | FilePathOrBuffer[bytes] = "",
+        path_or_buf: FilePath | WriteBuffer[str] | WriteBuffer[bytes] = "",
         sep: str = ",",
         cols: Sequence[Hashable] | None = None,
         index_label: IndexLabel | None = None,
@@ -57,21 +66,21 @@ class CSVFormatter:
         errors: str = "strict",
         compression: CompressionOptions = "infer",
         quoting: int | None = None,
-        line_terminator="\n",
+        lineterminator: str | None = "\n",
         chunksize: int | None = None,
         quotechar: str | None = '"',
         date_format: str | None = None,
         doublequote: bool = True,
         escapechar: str | None = None,
-        storage_options: StorageOptions = None,
-    ):
+        storage_options: StorageOptions | None = None,
+    ) -> None:
         self.fmt = formatter
 
         self.obj = self.fmt.frame
 
         self.filepath_or_buffer = path_or_buf
         self.encoding = encoding
-        self.compression = compression
+        self.compression: CompressionOptions = compression
         self.mode = mode
         self.storage_options = storage_options
 
@@ -82,7 +91,7 @@ class CSVFormatter:
         self.quotechar = self._initialize_quotechar(quotechar)
         self.doublequote = doublequote
         self.escapechar = escapechar
-        self.line_terminator = line_terminator or os.linesep
+        self.lineterminator = lineterminator or os.linesep
         self.date_format = date_format
         self.cols = self._initialize_columns(cols)
         self.chunksize = self._initialize_chunksize(chunksize)
@@ -100,7 +109,7 @@ class CSVFormatter:
         return self.fmt.decimal
 
     @property
-    def header(self) -> bool | Sequence[str]:
+    def header(self) -> bool | list[str]:
         return self.fmt.header
 
     @property
@@ -116,16 +125,16 @@ class CSVFormatter:
                 return [index_label]
         return index_label
 
-    def _get_index_label_from_obj(self) -> list[str]:
+    def _get_index_label_from_obj(self) -> Sequence[Hashable]:
         if isinstance(self.obj.index, ABCMultiIndex):
             return self._get_index_label_multiindex()
         else:
             return self._get_index_label_flat()
 
-    def _get_index_label_multiindex(self) -> list[str]:
+    def _get_index_label_multiindex(self) -> Sequence[Hashable]:
         return [name or "" for name in self.obj.index.names]
 
-    def _get_index_label_flat(self) -> list[str]:
+    def _get_index_label_flat(self) -> Sequence[Hashable]:
         index_label = self.obj.index.name
         return [""] if index_label is None else [index_label]
 
@@ -139,7 +148,7 @@ class CSVFormatter:
     def has_mi_columns(self) -> bool:
         return bool(isinstance(self.obj.columns, ABCMultiIndex))
 
-    def _initialize_columns(self, cols: Sequence[Hashable] | None) -> np.ndarray:
+    def _initialize_columns(self, cols: Iterable[Hashable] | None) -> np.ndarray:
         # validate mi options
         if self.has_mi_columns:
             if cols is not None:
@@ -160,7 +169,7 @@ class CSVFormatter:
 
     def _initialize_chunksize(self, chunksize: int | None) -> int:
         if chunksize is None:
-            return (100000 // (len(self.cols) or 1)) or 1
+            return (_DEFAULT_CHUNKSIZE_CELLS // (len(self.cols) or 1)) or 1
         return int(chunksize)
 
     @property
@@ -174,7 +183,7 @@ class CSVFormatter:
             "decimal": self.decimal,
         }
 
-    @property
+    @cache_readonly
     def data_index(self) -> Index:
         data_index = self.obj.index
         if (
@@ -184,6 +193,8 @@ class CSVFormatter:
             data_index = Index(
                 [x.strftime(self.date_format) if notna(x) else "" for x in data_index]
             )
+        elif isinstance(data_index, ABCMultiIndex):
+            data_index = data_index.remove_unused_levels()
         return data_index
 
     @property
@@ -209,8 +220,7 @@ class CSVFormatter:
                 raise ValueError(
                     f"Writing {len(self.cols)} cols but got {len(self.header)} aliases"
                 )
-            else:
-                return self.header
+            return self.header
         else:
             # self.cols is an ndarray derived from Index._format_native_types,
             #  so its entries are strings, i.e. hashable
@@ -242,11 +252,10 @@ class CSVFormatter:
             compression=self.compression,
             storage_options=self.storage_options,
         ) as handles:
-
             # Note: self.encoding is irrelevant here
             self.writer = csvlib.writer(
-                handles.handle,  # type: ignore[arg-type]
-                lineterminator=self.line_terminator,
+                handles.handle,
+                lineterminator=self.lineterminator,
                 delimiter=self.sep,
                 quoting=self.quoting,
                 doublequote=self.doublequote,
